import logging

from langchain.chat_models import init_chat_model
from langchain_core.messages import HumanMessage
from langchain_core.messages import SystemMessage

from .config import get_config

config = get_config()

log = logging.getLogger(__name__)


class Ai:
    def __init__(self):
        # google_genai google_vertexai
        provider = "google_genai" if config.env == "development" else "google_vertexai"
        try:
            self.model = init_chat_model(config.model, model_provider=provider)
        except:
            log.warning("Vertex not started")

    def invoke(self, messages):
        log.debug("message: %s", messages)
        response = self.model.invoke(messages)

        log.debug("response: %s", response)
        return response

    def structured_invoke(self, template, message):
        # Voglio ritornare i dati + le statistiche d'uso
        try:
            llm = self.model.with_structured_output(template, include_raw=True)
        except AttributeError:
            log.error("no GOOGLE API access")
            raise SystemExit

        response = llm.invoke(message)
        data = response.get("parsed")
        aim=response.get('raw')
        usage = aim.usage_metadata

        log.debug("response: %s", response)

        return data, usage
