from pathlib import Path
import os
from collections import defaultdict
from typing import Dict, List, Set
from dataclasses import dataclass, field

from gtagger.config import get_config
config = get_config()

@dataclass
class FileExplorer:
    _root: str
    _files: defaultdict = field(init=False)
    _extensions: set = field(init=False)

    def __post_init__(self):
        self._files = defaultdict(list)
        self._extensions = set(config.extension) | set(('.md' , '.json', '.ok') )

    @property
    def root(self) -> str:
        """Restituisce una copia 'pulita' del root path privata di asterischi"""
        if "*" in self._root:
            return Path(self._root).parent.resolve()
        return Path(self._root).resolve()

    @property
    def files(self) -> Dict[str, List[Path]]:
        """Restituisce una copia 'pulita' del dizionario dei file per estensione"""
        return dict(self._files)
    
    @property
    def extensions(self) -> Set[str]:
        """Restituisce una copia 'pulita' del dizionario dei file per estensione"""
        return self._extensions
    

    def traverse(self):
        """Percorre tutto l'albero a partire da una directory

        Crea un dzionario suddiviso per estensioni, tra quelle ammesse
        """

        # traverse è idempotente
        self._files = defaultdict(list)

        # Qui prendo TUTTI i files 
        if "*" in self._root:
            ###########base = Path(self.root).parent.resolve()
            files = self.root.rglob("*")
            # pattern = os.path.join(str(base), Path(self.root).name)

            # files = [
            #     os.path.abspath(f)
            #     for f in glob.iglob(pattern, recursive=True)
            #     if os.path.isfile(f)
            # ]
        else: 
            files = [self.root]
        
        for file in files:
            if file.is_file():
                suffix = file.suffix.lower()
                # Evitiamo il report eventualmente salvato in stats_collector.save_stats_as_json
                # Tutti i files con questo prefisso non vengono considerati (per accettare altri tipi di file
                # in futuro senza continuare a modificare codice)
                if file.name.startswith("__gtagger_"):
                    continue

                if suffix in self.extensions:
                    self._files[suffix].append(file)