# filename_builder.py
from datetime import date
from typing import Optional, List
from pathlib import Path
import logging
import re

from gtagger.model import Document
from gtagger.config import get_config
from pathlib import Path
import logging

config = get_config()
log = logging.getLogger(__name__)

class FilenameBuilder:
    @staticmethod
    def build_filename(documento: Document, tipo_evento: str, original_filename: Optional[str] = None) -> str:
        """
        Costruisce il nome del file secondo convenzioni custom.
        Il formato base è:
        <tipo_documento>-<tipo_evento>-<data>-<persone_evidenziate (opzionale)>
        """
        tipo_documento = documento.tipo_documento
        data_evento = documento.data_evento
        if not isinstance(data_evento, date):
            raise ValueError("data_evento deve essere un oggetto datetime.date")

        data_str = data_evento.strftime("%Y_%m_%d")
        info = Path(documento._filename_info.get("info","")).stem
        # Costruzione base del nome file
        info = re.sub(r"\W", "_", info)
        info = info.strip("_")
        parts = [tipo_documento, tipo_evento, data_str, info]

        # Aggiunta opzionale di persone (solo se richiesto per alcuni tipi)
        if tipo_documento in ("sp", "dis", "red", "int"):
            cognomi = [p.split()[-1].lower() for p in config.person]
            presenti = []
            lowered_filename = original_filename.lower() if original_filename else ""
            for cognome in cognomi:
                if cognome in lowered_filename:
                    presenti.append(cognome)

            if presenti:
                parts.append("_".join(presenti))

        filename = "-".join(filter(None, parts))
        log.debug(f"Nome costruito {parts} --> {filename}")
        return filename

    @staticmethod
    def rules():
        """
        In futuro qui si potrebbero caricare regole dinamiche da YAML o JSON
        per decidere cosa va incluso nel nome del file.
        """
        return {
            "add_people_for": ["sp", "dis"],
            "date_format": "%Y_%m_%d",
        }
