from datetime import datetime,timezone
import os
import random
import json
import time
import logging
from pathlib import Path
from typing import Optional

# richiede pip install httpx
import httpx
from http.cookies import SimpleCookie
from email.utils import parsedate_to_datetime

from .model import Correspondent, DocumentType, document_dict, event_dict
from .config import get_config

config = get_config()

#################from config import settings
#from log_module import get_logger

logger = logging.getLogger(__name__)

# URL della pagina di login di Paperless da mettere in config.py
PAPER_URL = config.paperless
API_TAGS = "/api/tags/"
API_TASKID = "/api/tasks/"
API_DOCUMENT = "/api/documents/post_document/"
API_CORRESPONDENTS = "/api/correspondents/"
API_CUSTOM_FIELDS = "/api/custom_fields/"
API_DOCUMENT_TYPES = "/api/document_types/"
API_PATCH = "/api/documents/"

headers = {
        'Authorization': 'Token ' + config.token
    }
####logger = get_logger(__name__, os.path.join(settings.get("working-path"),'paperless.log'),level=logging.DEBUG)

def generate_random_color():
    # Genera un colore a caso per i tag nuovi
    r = random.randint(0, 255)
    g = random.randint(0, 255)
    b = random.randint(0, 255)

    return f"#{r:02x}{g:02x}{b:02x}"


class Paperless:
    def __init__(self, token: Optional[str] = None):
        token = token or config.token
        ## TODO porto l'header qui dentro, ma devo togliere i metodi statici
        self.headers = {
            'Authorization': 'Token ' + token
        }

    def post_document(self,filename, payload):
        # NOTA: il token adesso è cablato, altrimenti ogni run deve fare una request
        # 1 - recupera il CSRF token (è una chiamata a funzione, che lo rinnova nel caso non lo  fosse)
        # 2 - Costruisci l'header
        # 3 - fai la chiamata
        # 4 - se return code == 200 controlla l'esito con il task_id restiruito

        # Il tag deve esistere, altrimenti status code 400 (ok, per il momento non abbiamo tag)
        #                                  text =  {"tags":["Invalid pk \"10000\" - object does not exist."]}
        # {
        # "id":25,
        # "task_id":"9f4848e3-507d-438c-845a-2e8df39e9d19",
        # "task_name":"consume_file",
        # "task_file_name":"Vinitaly 2024_Invito Giornalisti (002).pdf",
        # "date_created":"2025-05-09T06:25:48.524638Z",
        # "date_done":"2025-05-09T06:25:48.850663Z",
        # "type":"auto_task",
        # "status":"FAILURE",
        # "result":"Vinitaly 2024_Invito Giornalisti (002).pdf: Not consuming Vinitaly 2024_Invito Giornalisti (002).pdf: It is a duplicate of Vinitaly 2024_Invito Giornalisti (002) (#5).",
        # "acknowledged":false,
        # "related_document":"5",
        # "owner":3
        # }
        url = PAPER_URL + API_DOCUMENT
        try:
            with open(filename, 'rb') as f:
                files = {'document': f}

                # data = {
                #     'title': title,
                #     'tags': tags,
                # }


                with httpx.Client() as client:
                    response = client.post(url, headers=headers, files=files,data=payload)
                    if response.status_code == 200:
                        logger.info(f"Trying to upload document {filename} with name {payload['title']}")
                        return response.json()
                    else:
                        logger.error(f"Failed to upload document. Status code: {response.status_code}, {response.text}")
                        return None
        except FileNotFoundError:
            logger.error(f"File {filename} non trovato.")
            return None
        except Exception as e:
            logger.error(f"Errore imprevisto in post_document: {e}")
            return None


    def taskid_check(self, task_id):
        # Abbiamo deciso di non fare una gestine asincrona, ma un POST alla volta.
        # Si rimane in questo codice fino che lo status è definitivo
        url = PAPER_URL + API_TASKID
        params = {"task_id": f"{task_id}"}
        logger.debug(f"Check task: {task_id}")

        message = dict()

        while True:
            response = httpx.get(url, headers=headers, params=params)

            if response.status_code == 200:
                try:
                    json_data = response.json()
                    # Ocio che se è None va in errore ( e mi è successo!!!)
                    if not json_data or not isinstance(json_data, list):
                            logger.error(f"Risposta inattesa dal server per task_id {task_id}: {json_data}")
                            return None

                    result = json_data[0].get("status")
                    if result is None:
                        return None
                    if result == "SUCCESS" or result == "FAILURE":
                        message = json_data[0]
                        logger.debug(f"Task {task_id} status: {result}")
                        break
                except (IndexError, KeyError, ValueError) as e:
                    logger.error(f"Errore nel parsing della risposta JSON per task_id {task_id}: {e}")
                    return None
            else:
                logger.error(f"Errore {response.status_code}: {response.text}")
                message = {
                    "status": "FAILURE",
                    "status_code": response.status_code,
                }
                break
            time.sleep(3)
        return message

    @staticmethod
    def get_tags():
        response = httpx.get(PAPER_URL + API_TAGS, headers=headers)

        if response.status_code == 200:
            tags = { tag["slug"]:tag["id"] for  tag in response.json()["results"] }
            return tags
        else:
            logger.error(f"Errore {response.status_code}: {response.text}")
            return {}


    @staticmethod
    def post_tag(tag_name, color=None):
        # Nota: il valore "matching_algorithm" = 0 disabilita gli automatismi
        #       nell'assegnazione dei tags. Altrimenti può succedere che alcuni tag
        #       vengano assegnati da paperless in modo arbitrario
        #       Tutti i tag che usiamo devono essere così, se vogliamo il pieno
        #       controllo.

        data = {
            "name": f"{tag_name}",
            "is_insensitive": True,
            "matching_algorithm": 0
        }

        if color:
            data["color"]= f"{color}"

        response = httpx.post(PAPER_URL + API_TAGS, json=data, headers=headers)

        if response.status_code == 201:
            response_json = response.json()
            logger.debug(f"Risposta: {response_json}")
            return response_json.get("id")
        else:
            logger.error(f"Errore {response.status_code}: {response.text}")


    def load_tags(self):
        # TAG NORMALIZZATI
        # li preleviamo da un file json, risultato della classificazione IA
        #wp = os.path.join(settings.get("working-path"), settings.get("tag-file"))
        wp = os.path.join("/home/venom/Progetti/paperless-lab/importatore", "tags.json")
        classified_results = self.import_from_json(wp)

        # Estraggo i tags dall'oggetto
        # I tag non devono contenere spazi inizio e fine !!!!
        tags = set()
        for dizionario in classified_results:
            tags.update(dizionario["tags"])

        return tags


    def check_tags(self):
        # carica in un set la lista di tag normalizzata da IA
        document_tags = self.load_tags()
        logger.debug(f"tags normalizzati {document_tags}")

        # richiedi i tag di paperless in formato {chiave:valore}
        paperless_tags = self.get_tags()
        logger.debug(f"tags paperless {set(paperless_tags.keys())}")

        # differenza insiemistica: restano i tag che non esistono
        new_tags = document_tags - set(paperless_tags.keys())
        logger.debug(f"Trovati {len(new_tags)}"
                    f" nuovi tags {new_tags if len(new_tags) else ''}")

        # carica i nuovi tags in paperless
        if new_tags:
            logger.info(f"Aggiungo i nuovi tags: {new_tags} a paperless")
            for tag in new_tags:
                self.post_tag(tag, color=generate_random_color())

    @staticmethod
    def get_document_types():
        response = httpx.get(PAPER_URL + API_DOCUMENT_TYPES, headers=headers)

        if response.status_code == 200:
            doc_types = [DocumentType(**item) for item in response.json()["results"]]
            return doc_types
        else:
            logger.error(f"Errore {response.status_code}: {response.text}")
            return []


    @staticmethod
    def get_correspondents():
        response = httpx.get(PAPER_URL + API_CORRESPONDENTS, headers=headers)

        if response.status_code == 200:
            correspondents = [Correspondent(**item) for item in response.json()["results"]]
            return correspondents
        else:
            logger.error(f"Errore {response.status_code}: {response.text}")
            return []


    def get_custom_fields(self):
        response = httpx.get(PAPER_URL + API_CUSTOM_FIELDS, headers=headers)

        if response.status_code == 200:
            logger.debug("get_custom_fields todo")
            return response.json()["results"]
        else:
            logger.error(f"Errore {response.status_code}: {response.text}")
            return {}

    def get_document(self, id):
        logger.info(f"Get document ({PAPER_URL + API_PATCH + str(id)}/)")
        response = httpx.get(PAPER_URL + API_PATCH + str(id) + "/", headers=headers)
        if response.status_code == 200:
            return response.json()
        else:
            logger.error(f"Errore nella recupero del documento {id}")
            return {}

    def get_all_documents(self):
        logger.info(f"Get all documents ({PAPER_URL + API_PATCH})")
        response = httpx.get(PAPER_URL + API_PATCH, headers=headers)  
        if response.status_code == 200:
            return response.json()
        else:
            logger.error(f"Errore nella richiesta di tutti i documenti")
            return {}     

    def update_custom_fields(self, document_id, custom_fields):

        payload ={
            "custom_fields": [ {"field": i.id, "value":i.value} for i in custom_fields ]
        }

        logger.debug(f"update documento {document_id} con custom field {payload}")
        response = httpx.patch(PAPER_URL + API_PATCH + str(document_id) + "/", json=payload, headers=headers)
        if response.status_code == 200:
            logger.debug("update_custom_fields todo")
            return response.json()
        else:
            logger.error(f"Errore {response.status_code}: {response.text}")
            return {}

    def import_from_json(self,filename):
        # Importa i dati contenuti in un file json e li ritrna come dizionario
        try:
            with open(filename) as f:
                return json.load(f)
        except json.JSONDecodeError:
            logger.error(f"File {filename} non contiene json valido.")
        except FileNotFoundError:
            logger.error(f"File {filename} non trovato.")
        except Exception as e:
            logger.error(f"Errore: {e}")
        # Sa femo? Per il momento ritorno Non e non interrompo l'esecuzione
        return None


    def add_correspondents(self):
        correspondents = event_dict.keys()
        url = PAPER_URL + API_CORRESPONDENTS
        response = httpx.get(url, headers=headers)
        if response.status_code != 200:
            logger.error(f"Chiamata {url} ha restituito errore: {response.status_code}")
            return

        res = response.json().get('results')
        if res is None:
            logger.error(f"Chiamata {url} non ha restituito dati validi.")
            return

        cors = [dt.get('name') for dt in res]
        for correspondent in correspondents:
            if correspondent in cors:
                logger.debug(f"Corrispondente '{correspondent}' già presente")
                continue
            payload = {'name': correspondent,'matching_algorithm': 0}
            res = httpx.post( url, data=payload, headers=headers)
            if res.status_code == 201:
                logger.debug(f"({res.status_code})Aggiunto '{correspondent}' ai corrispondenti")
            else:
                logger.error(f"({res.status_code}) Errore durante l'aggiunta di '{correspondent}' ai corrispondenti")


    def add_document_types(self):
        document_types = [v for k,v in document_dict.items()]
        url = PAPER_URL + API_DOCUMENT_TYPES
        response = httpx.get(url, headers=headers)
        if response.status_code != 200:
            logger.error(f"Chiamata {url} ha restituito errore: {response.status_code}")
            return

        res = response.json().get('results')
        if res is None:
            logger.error(f"Chiamata {url} non ha restituito dati validi.")
            return

        doc_types = [dt.get('name') for dt in res]
        for document_type in document_types:
            if document_type in doc_types:
                logger.debug(f"Tipo documento '{document_type}' già presente")
                continue
            payload = {'name': document_type,'matching_algorithm': 0}
            res = httpx.post( url, data=payload, headers=headers)

            if res.status_code == 201:
                logger.debug(f"({res.status_code})Aggiunto '{document_type}' ai tipi di documento")
            else:
                logger.error(f"({res.status_code}) Errore durante l'aggiunta di '{document_type}' ai tipi di documento")


    def add_custom_fields(self):
        # FIXME Rischio consistenza. Andrebbe preso da model.py, ma manca il tipo di dati
        # che richiede una modifica in altri punti (da fare successivamente)
        property_to_name_map = {
            "persone citate": "string",
            "argomenti": "string",
            "anno": "integer",
            "tipo evento": "string",
            "persone evidenziate": "string",
            "relatori": "string",
            "nome file" : "string"
        }
        url = PAPER_URL + API_CUSTOM_FIELDS
        response = httpx.get(url, headers=headers)
        if response.status_code != 200:
            logger.error(f"Chiamata {url} ha restituito errore: {response.status_code}")
            return


        res = response.json().get('results')
        if res is None:
            logger.error(f"Chiamata {url} non ha restituito dati validi.")
            return
        cfs = [cf.get('name') for cf in res]

        for field, datatype in property_to_name_map.items():
            if field.title() in cfs:
                logger.debug(f"Campo personalizzato '{field}' già presente")
                continue

            payload = {'name': field.title(),'data_type': datatype}
            res = httpx.post( url, data=payload, headers=headers)
            if res.status_code == 201:
                logger.debug(f"({res.status_code})Aggiunto '{field}' ai campi personalizzati")
            else:
                logger.error(f"({res.status_code}) Errore durante l'aggiunta di '{field}' ai campi personalizzati")

    def update_document(self, document_id, document):

        #payload ={
        #    "custom_fields": [ {"field": i.id, "value":i.value} for i in custom_fields ]
        #}

        
        response = httpx.patch(PAPER_URL + API_PATCH + str(document_id) + "/", json=document, headers=headers)
        if response.status_code == 200:
            
            return response.json()
        else:
            logger.error(f"Errore {response.status_code}: {response.text}")
            return {}