import glob
import hashlib
import os
from collections import defaultdict, Counter
import logging
from langchain_core.messages import HumanMessage, SystemMessage
import re
import datetime
from typing import List, Optional, Dict, Any, Union
import json
import traceback


from gtagger.model import Document
from gtagger.config import get_config
from gtagger.llm_invoker import LLMInvoker

config = get_config()

log = logging.getLogger(__name__)

extensions = [".pdf", ".odt", ".txt", ".pptx", ".doc", ".xlsx", ".docx"]

class Tagger:
    def _remove_duplicate(self,files:List) -> List:
        # 
        priority = config.extension
        priority_map = {ext: i for i, ext in enumerate(priority)}

        grouped = defaultdict(list)

        for f in files:
            dir_path = os.path.dirname(f)
            name_no_ext = os.path.splitext(os.path.basename(f))[0]
            key = os.path.join(dir_path, name_no_ext)
            grouped[key].append(f)

        result = []
        for file_varianti in grouped.values():
            if len(file_varianti) == 1:
                result.append(file_varianti[0])
            else:
                # scegli quello con l’estensione migliore
                best = min(file_varianti, key=lambda f: priority_map.get(os.path.splitext(f)[1].lower(), float('inf')))
                log.info(f"Rilevati files multipli: {file_varianti}")
                log.info(f"Selezionato {best}")
                result.append(best)

        return result

    def traverse(self, path: str, source: bool=True, result: bool=True):
        #path = '/mnt/disk/Verona*'
        # path = '/etc/resolv.conf'
        files = []

        # Qui prendo TUTTI i files 
        if isinstance(path, str) and "*" in path:
            from pathlib import Path
            base = Path(path).parent.resolve()
            pattern = os.path.join(str(base), Path(path).name)

            files.extend(os.path.abspath(f) for f in glob.iglob(pattern, recursive=True) if os.path.isfile(f))
            
        else: 
            files.append(os.path.abspath(path))
        
        documents = []
        # Devo trovare tutti i documenti ammessi, eliminando i duplicati
        if source:
            filtered_files = [f for f in files if os.path.splitext(f)[1].lower() in config.extension]
            documents = self._remove_duplicate(filtered_files)

        # Trovo tutti i files elaborati
        if result:
            valid_extensions = (".md", ".json")
            evaluated_files = [f for f in files if os.path.splitext(f)[1].lower() in valid_extensions]
            documents.extend(evaluated_files)
        
        return documents
    

    

    def cf_stats(self, custom_fields):
        cf_statistics = defaultdict(Counter)

        for custom_field in custom_fields:
            field = custom_field.custom_field_to_dict()
            for key, value in field.items():
                if isinstance(value, (list, set)):  # Unifica list e set
                    cf_statistics[key].update(value)
                else:  # Considera tutto il resto (int, str, ecc.)
                    cf_statistics[key][value] += 1

        for k in cf_statistics:
            sorted = cf_statistics[k].most_common()
            cf_statistics[k] = sorted

        return cf_statistics
    
