import logging
import argparse
import configargparse
#import sys

#_config = None


def configure_logging(debug_flag: bool, environment: str = None, log_file: str = None):
    # Livello globale
    logging.basicConfig(
        force=True,
        handlers=[]
    )
    root_logger = logging.getLogger()
    root_logger.setLevel(logging.WARNING)

    # log gtagger
    app_logger = logging.getLogger("nlsearch")
    app_logger.setLevel(logging.DEBUG if debug_flag else logging.INFO)
    formatter = logging.Formatter("%(asctime)s [%(levelname)s] %(name)s %(message)s")

    # Crea e aggiungi handler per la console
    console_handler = logging.StreamHandler()
    console_handler.setFormatter(formatter)
    app_logger.addHandler(console_handler)

    # Aggiungi handler per il file se specificato
    if environment == "development" and log_file:
        file_handler = logging.FileHandler(log_file)
        file_handler.setFormatter(formatter)
        app_logger.addHandler(file_handler)

    if debug_flag:
        import jurigged
        logging.getLogger("watchdog").setLevel(logging.INFO)
        jurigged.watch(pattern="*gtagger*.py")



def get_config() -> argparse.Namespace:
    parser = configargparse.get_argument_parser(
        #config_file_parser_class=configargparse.YAMLConfigFileParser,
        #formatter_class=configargparse.ArgumentDefaultsHelpFormatter,
    )
    parser.add_argument(
        "-c", "--config", required=False, is_config_file=True, help="config file path"
    )

    parser.add_argument(
        "--config-save",
        required=False,
        is_write_out_config_file_arg=True,
        help="config file path",
    )

    parser.add_argument("files", nargs="*", default=[], help="list of documents")
    parser.add_argument("--debug", action="store_true", default=False, help="debug")
    parser.add_argument("--semantic-chunker", action="store_true", default=False, help="semantic chunker")
    
    parser.add_argument("--dry-run", action="store_true", default=False, help="Check only")
    parser.add_argument("--env", choices=["development", "production"], default="development", help="Running mode(development o production)")
    parser.add_argument("--log-file", default=None, help="Log file")


    parser.add_argument("--db", default="./runtime", help="ChromaDB directory")
    parser.add_argument("--semantic-model", default="LaBSE", help="model default: LaBSE")

    parser.add_argument("--query", default="", help="query to search for")
    parser.add_argument("--n-results", type=int, default=5, help="Number of results to return")
    parser.add_argument("--max-metric", type=float, default=1.0, help="Max metric to search for")
    parser.add_argument("--chunk-size", action="append", type=int, default=[], help="chunk size, default 30")
    parser.add_argument("--chunk-overlap", action="append", type=int, default=[], help="chunk overlap, default 10")

    parser.add_argument("--reset", default=False, help="Reset ChromaDB")
    parser.add_argument("--delete", default=False, help="Delete ChromaDB Collection")

    parser.add_argument(
        "--collection",
        type=str,
        default="default",
        help="Collection name",
    )

    _config, _ = parser.parse_known_args()


    configure_logging(_config.debug, _config.env, _config.log_file)

    return _config


config = get_config()
